/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package it.eng.spago4q.presentation.tags;

import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

import java.io.IOException;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/*
The tag creates an HTML form that lets the user choose a scale.
 */
public class AddingFormListTag extends TagSupport
{
	protected HttpServletRequest httpRequest = null;
	protected DataConnection dataConnection=null;
	protected SQLCommand sqlCommand = null;
	protected DataResult dataResult = null;

	protected String statementName=null;
	protected String formName= null;
	protected String whereClauseName=null;
	protected String whereClauseValue=null;
	protected Vector<String> assignedIds=null;


	@Override
	public int doStartTag() throws JspException {

		try {

			httpRequest = (HttpServletRequest) pageContext.getRequest();
			DataConnectionManager dataConnectionManager=DataConnectionManager.getInstance();
			dataConnection=dataConnectionManager.getConnection("spago4q");
			String statement = SQLStatements.getStatement(statementName);
			if(whereClauseName!=null && whereClauseValue!=null && !whereClauseName.equalsIgnoreCase("") && !whereClauseValue.equalsIgnoreCase(""))
			{
				statement=statement+" WHERE "+whereClauseName+"="+whereClauseValue;
			}
			sqlCommand = dataConnection.createSelectCommand(statement);
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			StringBuffer htmlStream = new StringBuffer();
			htmlStream.append("<select name=\""+formName+"\" size=\"1\">");

			// this Boolean checks that at least one option is set, if none is than no select code is returned
			Boolean noone=new Boolean(true);
			// an option for each row in dt_scale
			for(int i=0;i<scrollableDataResult.getRowsNumber();i++){
				DataRow dataRow=scrollableDataResult.getDataRow();
				DataField idF=dataRow.getColumn("id");
				DataField nameF=dataRow.getColumn("name");
				String id=idF.getStringValue();
				String name=nameF.getStringValue();

				//set the option only if the id is not already assigned
				if(!(assignedIds.contains(id)) && !(id.equalsIgnoreCase("0"))){
					noone=false;
					htmlStream.append("<option value=\""+id+"\"");
					htmlStream.append(">"+name+"</option>");
				}
			}
			htmlStream.append("</select>");
			scrollableDataResult.close();
			if(noone.equals(false)){
				pageContext.getOut().print(htmlStream);
			}
			else{
				htmlStream = new StringBuffer();
				htmlStream.append("<div class=\"div_detail_error_form\">No other option avalaible</div>");
				pageContext.getOut().print(htmlStream);
			}	
		} catch (EMFInternalError e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} 		
		finally{
			try {
				dataConnection.close();
			} catch (EMFInternalError e) {
				e.printStackTrace();
			}
		}
		return SKIP_BODY;
	}


	public String getStatementName() {
		return statementName;
	}


	public void setStatementName(String statementName) {
		this.statementName = statementName;
	}


	public String getFormName() {
		return formName;
	}


	public void setFormName(String formName) {
		this.formName = formName;
	}


	public String getWhereClauseName() {
		return whereClauseName;
	}


	public void setWhereClauseName(String whereClauseName) {
		this.whereClauseName = whereClauseName;
	}

	public String getWhereClauseValue() {
		return whereClauseValue;
	}


	public void setWhereClauseValue(String whereClauseValue) {
		this.whereClauseValue = whereClauseValue;
	}


	public Vector<String> getAssignedIds() {
		return assignedIds;
	}


	public void setAssignedIds(Vector<String> assignedIds) {
		this.assignedIds = assignedIds;
	}



} // public class ListTag extends TagSupport






