/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.presentation.tags;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

/*
 Extends List Tag to create a List there only the last element can be deleted 
 */

public class OrderScaleValueListTag extends ListTag {

	protected String idC;

	
	public String getIdC() {
		return idC;
	}

	public void setIdC(String idC) {
		this.idC = idC;
	}

	@Override
	protected void makeForm() throws JspException {

		String titleCode = (String) _layout.getAttribute("TITLE");
		SourceBean buttonsSB = (SourceBean) _layout.getAttribute("BUTTONS");
		List buttons = buttonsSB.getContainedSourceBeanAttributes();

		// it wants the name of the scale of wich you want to modify the values
		httpRequest = (HttpServletRequest) pageContext.getRequest();
		RequestContainer requestContainer=RequestContainer.getRequestContainer();
		SourceBean ss=requestContainer.getServiceRequest();
		String name="";
		if(ss.getAttribute("NAME")!=null){
			name=(String)ss.getAttribute("NAME");}


		if (titleCode != null && buttons.size() > 0) {
			String title = msgBuilder.getMessage(titleCode, "messages", httpRequest);
			_htmlStream.append(" <table class=\"header-table-portlet-section\">\n");
			_htmlStream.append("	<tr class='header-row-portlet-section'>\n");
			_htmlStream.append("			<td class=\"header-title-column-portlet-section\" style=\"vertical-align:middle;padding-left:5px;\" >" + title + ": "+name+ "</td>\n");
			_htmlStream.append("			<td class=\"header-empty-column-portlet-section\">&nbsp;</td>\n");
			_htmlStream.append(				makeButton(buttons) + "\n");
			_htmlStream.append("	</tr>\n");
			_htmlStream.append(" </table>\n");
		}

		makeColumns();
		makeRows();
		makeNavigationButton();

	} // public void makeForm()

	@Override
	protected void makeRows() throws JspException {
		List rows = _content.getAttributeAsList("PAGED_LIST.ROWS.ROW");

		// js function for item action confirm
		String confirmCaption = msgBuilder.getMessage("ListTag.confirmCaption", "messages", httpRequest);
		//String confirmCaption = PortletUtilities.getMessage("ListTag.confirmCaption", "messages");
		_htmlStream.append(" <script>\n");
		_htmlStream.append("	function actionConfirm(message, url){\n");
		_htmlStream.append("		if (confirm('" + confirmCaption + " ' + message + '?')){\n");
		_htmlStream.append("			location.href = url;\n");
		_htmlStream.append("		}\n");
		_htmlStream.append("	}\n");
		_htmlStream.append(" </script>\n");

		boolean alternate = false;
		String rowClass;


		String idLast=isLast();

		for(int i = 0; i < rows.size(); i++) 
		{
			SourceBean row = (SourceBean) rows.get(i);

			rowClass = (alternate) ? "portlet-section-alternate" : "portlet-section-body";
			alternate = !alternate;    

			_htmlStream.append(" <tr class='portlet-font'>\n");
			for (int j = 0; j < _columns.size(); j++) {
				String nameColumn = (String) ((SourceBean) _columns.elementAt(j)).getAttribute("NAME");
				Object fieldObject = row.getAttribute(nameColumn);
				String field = null;
				if (fieldObject != null)
					field = fieldObject.toString();
				else
					field = "&nbsp;";
				// if an horizontal-align is specified it is considered, otherwise the defualt is align='left'
				String align = (String) ((SourceBean) _columns.elementAt(j)).getAttribute("horizontal-align");
				if (align == null || align.trim().equals("")) align = "left";
				_htmlStream.append(" <td class='" + rowClass + "' style='vertical-align:middle;text-overflow:ellipsis;text-align:" + align + ";' >" + field + "</td>\n");
			} 


			SourceBean captionsSB = (SourceBean) _layout.getAttribute("CAPTIONS");
			List captions = captionsSB.getContainedSourceBeanAttributes();
			Iterator iter = captions.iterator();
			Boolean last=new Boolean(false);
			Boolean onDelete=new Boolean(false);

			while (iter.hasNext()) {
				SourceBeanAttribute captionSBA = (SourceBeanAttribute)iter.next();
				SourceBean captionSB = (SourceBean)captionSBA.getValue();
				SourceBean conditionsSB = (SourceBean) captionSB.getAttribute("CONDITIONS");
				boolean conditionsVerified = verifyConditions(conditionsSB, row);
				if (!conditionsVerified) {
					// if conditions are not verified puts an empty column
					_htmlStream.append(" <td width='40px' class='" + rowClass + "' >&nbsp;</td>\n");
					continue;
				}
				List parameters = captionSB.getAttributeAsList("PARAMETER");


				//I want to draw the delete button only on the last row
				String currentId=String.valueOf(row.getAttribute("idv"));
				last=(currentId.equalsIgnoreCase(idLast));
				
				onDelete=captionSB.getName().equalsIgnoreCase("DELETE_CAPTION");
				if (parameters == null || parameters.size() == 0 || (onDelete && !last)) {
					// if there are no parameters puts an empty column
					_htmlStream.append(" <td width='40px' class='" + rowClass + "' >&nbsp;</td>\n");
				} else {
					HashMap paramsMap = getParametersMap(parameters, row);
					String img = (String)captionSB.getAttribute("image");
					String labelCode = (String)captionSB.getAttribute("label");
					//String label = PortletUtilities.getMessage(labelCode, "messages");
					String label = msgBuilder.getMessage(labelCode, "messages", httpRequest);
					String buttonUrl = createUrl(paramsMap);
					boolean confirm = false;
					if (captionSB.getAttribute("confirm") != null &&
							((String)captionSB.getAttribute("confirm")).equalsIgnoreCase("TRUE")){
						confirm = true;
					}
					_htmlStream.append(" <td width='40px' class='" + rowClass + "'>\n");
					if (confirm){
						_htmlStream.append("     <a href='javascript:actionConfirm(\"" + label + "\", \"" + buttonUrl+ "\");'>\n");	
					}else{
						_htmlStream.append("     <a href='"+buttonUrl+"'>\n");	
					}
					_htmlStream.append("			<img title='"+label+"' alt='"+label+"' src='"+urlBuilder.getResourceLink(httpRequest, img)+"' />\n");
					_htmlStream.append("     </a>\n");
					_htmlStream.append(" </td>\n");
				}
			}
			_htmlStream.append(" </tr>\n");
		}

		_htmlStream.append(" </table>\n");

	}

	protected String isLast(){
		String id="";
		DataConnection dataConnection=null;
		ScrollableDataResult scrollableDataResult=null;
		DataResult dataResult=null;
		DataConnectionManager dataConnectionManager=null;
		
		try {
			dataConnectionManager=DataConnectionManager.getInstance();
			dataConnection=dataConnectionManager.getConnection("spago4q");
			
			String statement="select count(id_scale_value) as countId from dt_scale_value where id_scale="+idC;

			SQLCommand sqlCommand = dataConnection.createSelectCommand(statement);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();

			DataRow dataRow=scrollableDataResult.getDataRow();
			DataField idM=dataRow.getColumn("countId");
			id=idM.getStringValue();

		} catch (EMFInternalError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		finally{
			try {
				dataConnection.close();
				scrollableDataResult.close();
			} catch (EMFInternalError e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
			}		
	
		return id;
	}
}
