/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.presentation.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago4q.presentation.tags.treehtmlgenerators.ITreeHtmlGenerator;
import it.eng.spago4q.utilities.ChannelUtilities;
import it.eng.spago4q.utilities.Spago4QConstants;
import it.eng.spago4q.utilities.Spago4QTracer;

import java.io.IOException;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Creates the tag for tree objects
 * 
 * @author sulis
 */
public class TreeObjectsTag extends TagSupport {

	private String moduleName_ = null;
	private String htmlGeneratorClass_ = null;
	private String treeName_ = null;
	HttpServletRequest httpRequest_ = null;
	private String attributeToRender_ = null;
	private List list_ = null;

	/**
	 * Starting tag
	 */
	public int doStartTag() throws JspException {
		httpRequest_ = (HttpServletRequest) pageContext.getRequest();
		RequestContainer requestContainer = ChannelUtilities.getRequestContainer(httpRequest_);
		SourceBean serviceRequest = requestContainer.getServiceRequest();
		List objectsList = null;

		if (list_ != null){
			objectsList = list_;
		} else if (attributeToRender_ == null || attributeToRender_.trim().equals("")) {
		} else {
		}


		//TODO da capire se serve
		String initialPath = null;


		ITreeHtmlGenerator gen = null;
		try{
			gen = (ITreeHtmlGenerator)Class.forName(htmlGeneratorClass_).newInstance();
		} catch(Exception e) {
			return -1;
		}

		StringBuffer htmlStream = null;
		if(treeName_==null) {
			htmlStream = gen.makeTree(objectsList, httpRequest_, initialPath);
		} else {
			htmlStream = gen.makeTree(objectsList, httpRequest_, initialPath, treeName_);
		}

		try {
			pageContext.getOut().print(htmlStream);
		} catch(IOException ioe) {
			Spago4QTracer.major(Spago4QConstants.NAME_MODULE, this.getClass().getName(), 
					"doStartTag", "cannot start object tree tag: IOexception occurred",ioe);
		}
		return SKIP_BODY;
	}


	/**
	 * ending tag
	 */
	public int doEndTag() throws JspException {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "TitleTag::doEndTag:: invocato");
		return super.doEndTag();
	}

	/**
	 * @return the module name
	 */
	public String getModuleName() {
		return moduleName_;
	}
	/**
	 * @param moduleName the module name to set
	 */
	public void setModuleName(String moduleName) {
		this.moduleName_ = moduleName;
	}
	/**
	 * @return the html generator class
	 */
	public String getHtmlGeneratorClass() {
		return htmlGeneratorClass_;
	}
	/**
	 * @param htmlGeneratorClass the html generator class to set
	 */
	public void setHtmlGeneratorClass(String htmlGeneratorClass) {
		this.htmlGeneratorClass_ = htmlGeneratorClass;
	}


	public String getTreeName() {
		return treeName_;
	}


	public void setTreeName(String treeName) {
		this.treeName_ = treeName;
	}


	public String getAttributeToRender() {
		return attributeToRender_;
	}


	public void setAttributeToRender(String attributeToRender) {
		this.attributeToRender_ = attributeToRender;
	}


	public List getList_() {
		return list_;
	}


	public void setList_(List list_) {
		this.list_ = list_;
	}


}
