/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.presentation.tags.treehtmlgenerators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.list.basic.impl.DelegatedBasicListService;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago4q.utilities.Spago4QConstants;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class AssessStructureTreeHtmlGenerator extends StructureTreeHtmlGenerator  {

	int af_ = -1;
	int ac_ = -1;
	int at_ = -1;
	int ap_ = -1;

	int id_af_ = -1;
	int id_ac_ = -1;
	int id_at_ = -1;
	int id_ap_ = -1;

	@Override
	protected String getTreeName(List objectsList) {
		try {
			SourceBean sb = (SourceBean) objectsList.get(0);
			return (String) sb.getAttribute("af_name");
		} catch (Exception e) {
			return "";
		}
	}

	@Override
	protected void addItems(StringBuffer htmlStream, List oldItems,
			String initialPath) {

		int index = -1;

		List items=completeList(oldItems);

		Iterator it = items.iterator();	

		while (it.hasNext()) {

			SourceBean item = (SourceBean) it.next();

			int af=-1;
			int ac=-1;
			int at=-1;
			int ap=-1;

						
			if(item.getAttribute("af")!=null)
				af=(Integer.valueOf((String)item.getAttribute("af"))).intValue();

			if(item.getAttribute("ac")!=null)
				ac=(Integer.valueOf((String)item.getAttribute("ac"))).intValue();

			if(item.getAttribute("at")!=null)
				at=(Integer.valueOf((String)item.getAttribute("at"))).intValue();
			
			if(item.getAttribute("ap")!=null)
				ap=(Integer.valueOf((String)item.getAttribute("ap"))).intValue();

			index = index + 1;
			String name = "";
			String itemType = "";
			boolean isInitialPath=false;
			boolean isFinalPath=false;

			int parentId = DTREE_ROOT_ID;

			if (af_ == -1 || af != af_){
				af_ = af;
				ac_ = -1;
				at_ = -1;
				ap_ = -1;
				id_af_ = index;
				parentId = -1;
				itemType = "af";
				name = (String) item.getAttribute("af_name");
				isInitialPath=true;
			}

			else if (ac_ == -1 || ac != ac_){
				ac_ = ac;
				at_ = -1;
				ap_ = -1;
				id_ac_ = index;
				parentId = id_af_;
				itemType = "ac";
				name = (String) item.getAttribute("ac_name");
			}

			else if (at_ == -1 || at != at_){
				at_ = at;
				ap_ = -1;
				id_at_ = index;
				parentId = id_ac_;
				itemType = "at";
				name = (String) item.getAttribute("at_name");
			}

			else if (ap_ == -1 || ap != ap_){
				ap_ = ap;
				id_ap_ = index;
				parentId = id_at_;
				itemType = "ap";
				name = (String) item.getAttribute("ap_name");
				isFinalPath=true;
			}

			try {
				item.updAttribute("itemType", itemType);
			} catch (SourceBeanException e) {
				e.printStackTrace();
			}

			addItemForJSTree(htmlStream, item, index, parentId, name, isInitialPath, isFinalPath);


		}
	}

	protected String createDetailObjectLink(SourceBean item) {
		HashMap editUrlParMap = new HashMap();
		editUrlParMap.put(Spago4QConstants.PAGE, "AssessmentFrameworkTreePage");
		editUrlParMap.put(Spago4QConstants.MODULE, "ListAssessmentFrameworkTreeModule");
		editUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_SELECT");
		editUrlParMap.putAll(getItemParams(item));
		String editUrl = urlBuilder.getUrl(httpRequest, editUrlParMap);
		return editUrl;
	}

	protected String createAddObjectLink(SourceBean item) {
		HashMap addUrlParMap = new HashMap();
		addUrlParMap.put(Spago4QConstants.PAGE, "AssessmentFrameworkTreePage");
		addUrlParMap.put(Spago4QConstants.MODULE, "ListAssessmentFrameworkTreeModule");
		addUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_NEW");
		addUrlParMap.putAll(getItemParams(item));
		String addUrl = urlBuilder.getUrl(httpRequest, addUrlParMap);
		return addUrl;
	}

	protected String createDeleteObjectLink(SourceBean item) {
		HashMap deleteUrlParMap = new HashMap();
		deleteUrlParMap.put(Spago4QConstants.PAGE, "AssessmentFrameworkTreePage");
		deleteUrlParMap.put(Spago4QConstants.MODULE, "ListAssessmentFrameworkTreeModule");
		deleteUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, DelegatedBasicListService.LIST_DELETE);
		deleteUrlParMap.putAll(getItemParamsDelete(item));
		String addUrl = urlBuilder.getUrl(httpRequest, deleteUrlParMap);
		return addUrl;
	}




	@Override
	protected Map getItemParams(SourceBean item) {
		Map m = new HashMap();

		Integer af=-1;
		Integer ac=-1;
		Integer at=-1;
		Integer ap=-1;
		String af_name="";
		String ac_name="";
		String at_name="";
		String ap_name="";
		String ac_code="";
		String at_code="";
		String ap_code="";

		if(item.getAttribute("af")!=null)
			af = Integer.valueOf((String) item.getAttribute("af")).intValue();

		if(item.getAttribute("af_name")!=null)
			af_name = (String) item.getAttribute("af_name");

		if(item.getAttribute("ac")!=null)
			ac = Integer.valueOf((String) item.getAttribute("ac")).intValue();

		if(item.getAttribute("ac_name")!=null)
			ac_name = (String) item.getAttribute("ac_name");

		if(item.getAttribute("ac_code")!=null)
			ac_code = (String) item.getAttribute("ac_code");

		if(item.getAttribute("at")!=null)
			at = Integer.valueOf((String) item.getAttribute("at")).intValue();

		if(item.getAttribute("at_name")!=null)	
			at_name = (String) item.getAttribute("at_name");

		if(item.getAttribute("at_code")!=null)	
			at_code = (String) item.getAttribute("at_code");

		if(item.getAttribute("ap")!=null)	
			ap = Integer.valueOf((String) item.getAttribute("ap")).intValue();

		if(item.getAttribute("ap_name")!=null)
			ap_name = (String) item.getAttribute("ap_name");

		if(item.getAttribute("ap_code")!=null)	
			ap_code = (String) item.getAttribute("ap_code");

		String itemType = (String) item.getAttribute("itemType");
		m.put("af", af.intValue());
		m.put("af_name", af_name);
		m.put("ac", ac.intValue());
		m.put("ac_name", ac_name);
		m.put("ac_code", ac_code);
		m.put("at", at.intValue());
		m.put("at_name", at_name);
		m.put("at_code", at_code);
		m.put("ap", ap.intValue());
		m.put("ap_name", ap_name);
		m.put("ap_code", ap_code);
		m.put("type", itemType);

		return m;
	}

	@Override
	public Map getItemParamsDelete(SourceBean item) {
		Map m = new HashMap();

		Integer af=-1;
		Integer ac=-1;
		Integer at=-1;
		Integer ap=-1;

		if(item.getAttribute("af")!=null)
			af = Integer.valueOf((String) item.getAttribute("af")).intValue();
		if(item.getAttribute("ac")!=null)	
			ac = Integer.valueOf((String) item.getAttribute("ac")).intValue();
		if(item.getAttribute("at")!=null)	
			at = Integer.valueOf((String) item.getAttribute("at")).intValue();
		if(item.getAttribute("ap")!=null)	
			ap = Integer.valueOf((String) item.getAttribute("ap")).intValue();

		String itemType = (String) item.getAttribute("itemType");

		m.put("af", af.intValue());
		m.put("ac", ac.intValue());
		m.put("at", at.intValue());
		m.put("ap", ap.intValue());
		m.put("type", itemType);
		return m;
	}


	public List completeList(List items){

		String af="-1";
		String ac="-1";
		String at="-1";
		String ap="-1";

		String af_name="";
		String ac_name="";
		String ac_code="";
		String at_name="";
		String at_code="";
		String ap_name="";
		String ap_code="";

		String p=new String();
		String element=new String();

		PaginatorIFace paginator = new GenericPaginator();

		List completed=new ArrayList();
		Vector<String> path=new Vector<String>();
		Iterator it = items.iterator();


		while (it.hasNext()) {

			af="-1";
			ac="-1";
			at="-1";
			ap="-1";

			af_name="";
			ac_name="";
			ac_code="";
			at_name="";
			at_code="";
			ap_name="";
			ap_code="";


			SourceBean item = (SourceBean) it.next();
			if(item.getAttribute("af")!=null)
			{
				af = (String.valueOf(item.getAttribute("af")));
				if(item.getAttribute("af_name")!=null)
					af_name=(String)item.getAttribute("af_name");

				p=af+"-0-0-0";

				if(!path.contains(p)){

					element=writeRow(af, af_name, "0", "", "", "0", "", "", "0", "", "");
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);


					completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					path.add(p);

				}
			}

			if(item.getAttribute("ac")!=null)
			{
				ac = (String.valueOf(item.getAttribute("ac")));
				if(item.getAttribute("ac_name")!=null)
					ac_name=(String)item.getAttribute("ac_name");
				if(item.getAttribute("ac_code")!=null)
					ac_code=(String)item.getAttribute("ac_code");

				p=af+"-"+ac+"-0-0";

				if(!path.contains(p)){
					
					element=writeRow(af, af_name, ac, ac_name, ac_code, "0", "", "", "0", "", "");

					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
			
					path.add(p);

				}
			}

			if(item.getAttribute("at")!=null)
			{
				at = (String.valueOf(item.getAttribute("at")));
				if(item.getAttribute("at_name")!=null)
					at_name=(String)item.getAttribute("at_name");
				if(item.getAttribute("at_code")!=null)
					at_code=(String)item.getAttribute("at_code");

				p=af+"-"+ac+"-"+at+"-0";

				if(!path.contains(p)){

					element=writeRow(af, af_name, ac, ac_name, ac_code, at, at_name, at_code, "0", "", "");
					
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					path.add(p);

				}
			}			

			if(item.getAttribute("ap")!=null)
			{
				ap = (String.valueOf(item.getAttribute("ap")));
				if(item.getAttribute("ap_name")!=null)
					ap_name=(String)item.getAttribute("ap_name");
				if(item.getAttribute("ap_code")!=null)
					ap_code=(String)item.getAttribute("ap_code");

				p=af+"-"+ac+"-"+at+"-"+ap;

				if(!path.contains(p)){


					element=writeRow(af, af_name, ac, ac_name, ac_code, at, at_name, at_code, ap, ap_name, ap_code);
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				
					path.add(p);

				}
			}
		}

		return completed;

	}



	
	public String writeRow(String af, String af_name, String ac, String ac_name, String ac_code,
			String at, String at_name, String at_code, String ap, String ap_name, String ap_code){

		String s=new String("");

			s=s+"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";

		s=s+"<ROW af=\""+af+"\" af_name=\""+af_name+"\"" + 
		" ac=\""+ac+"\" ac_name=\""+ac_name+"\" ac_code=\""+ac_code+"\""+  
		" at=\""+at+"\" at_name=\""+at_name+"\" at_code=\""+at_code+"\""+  
		" ap=\""+ap+"\" ap_name=\""+ap_name+"\" ap_code=\""+ap_code+"\"/>\n";  

		return s;
	}

}

