/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.presentation.tags.treehtmlgenerators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.list.basic.impl.DelegatedBasicListService;
import it.eng.spago4q.utilities.Spago4QConstants;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class MeasureModelTreeHtmlGenerator extends ModelTreeHtmlGenerator {
	int id_=-1;
	int area_ = -1;
	int mf_ = -1;
	int in_ = -1;
	int mc_ = -1;
	int ma_ = -1;

	int id_mm_=-1;
	int id_area_ = -1;
	int id_mf_ = -1;
	int id_in_ = -1;
	int id_mc_ = -1;
	int id_ma_ = -1;

	@Override
	protected String getTreeName(List objectsList) {
		try {
			SourceBean sb = (SourceBean) objectsList.get(0);
			return (String) sb.getAttribute("mm_name");
		} catch (Exception e) {
			return "";
		}
	}

	@Override
	protected void addItems(StringBuffer htmlStream, List items,
			String initialPath) {

		int index = -1;
		Iterator it = items.iterator();

		while (it.hasNext()) {
			SourceBean item = (SourceBean) it.next();

			int id=((Integer) item.getAttribute("id")).intValue();
			int area = ((Integer) item.getAttribute("id_area")).intValue();
			int mf = ((Integer) item.getAttribute("id_measurement_framework")).intValue();
			int in = ((Integer) item.getAttribute("id_information_need")).intValue();
			int mc = ((Integer) item.getAttribute("id_measurable_concept")).intValue();
			int ma = ((Integer) item.getAttribute("id_measurable_attribute")).intValue();
			int kpi = ((Integer) item.getAttribute("id_kpi")).intValue();

			index = index + 1;
			String name = "";
			String itemType = "";
			boolean isInitialPath=false;
			boolean isFinalPath=false;
			boolean bkpi = false;
			int parentId = DTREE_ROOT_ID;
			if(id_==-1 || id!=id_){
				id_=id;
				area_ = -1;
				mf_ = -1;
				in_ = -1;
				mc_ = -1;
				ma_ = -1;
				id_mm_ = index;
				parentId = -1;
				itemType = "mm";
				name = (String) item.getAttribute("mm_name");
				isInitialPath=true;
			}
			else if (area_ == -1 || area != area_) {
				area_ = area;
				mf_ = -1;
				in_ = -1;
				mc_ = -1;
				ma_ = -1;
				id_area_ = index;
				parentId = DTREE_ROOT_ID;
				itemType = "area";
				name = (String) item.getAttribute("area_name");
			}

			else if (mf_ == -1 || mf != mf_){
				mf_ = mf;
				in_ = -1;
				mc_ = -1;
				ma_ = -1;
				id_mf_ = index;
				parentId = id_area_;
				itemType = "mf";
				name = (String) item.getAttribute("mf_name");
			}

			else if (in_ == -1 || in != in_){
				in_ = in;
				mc_ = -1;
				ma_ = -1;
				id_in_ = index;
				parentId = id_mf_;
				itemType = "in";
				name = (String) item.getAttribute("in_name");
			}

			else if (mc_ == -1 || mc != mc_){
				mc_ = mc;
				ma_ = -1;
				id_mc_ = index;
				parentId = id_in_;
				itemType = "mc";
				name = (String) item.getAttribute("mc_name");
			}

			else if (ma_ == -1 || ma != ma_){
				ma_ = ma;
				id_ma_ = index;
				parentId = id_mc_;
				itemType = "ma";
				name = (String) item.getAttribute("ma_name");
				isFinalPath=true;
			}

			try {
				item.updAttribute("itemType", itemType);
			} catch (SourceBeanException e) {
				e.printStackTrace();
			}

			bkpi = (kpi != 0) ? true : false;

			addItemForJSTree(htmlStream, item, index, parentId, name, isInitialPath, isFinalPath, bkpi);


		}
	}

	protected String createDetailObjectLink(SourceBean item) {
		HashMap editUrlParMap = new HashMap();
		editUrlParMap.put(Spago4QConstants.PAGE, "MeasureModelKPIPage");
		editUrlParMap.put(Spago4QConstants.MODULE, "ListMeasureModelKPIModule");
		editUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_SELECT");
		editUrlParMap.putAll(getItemParams(item));
		String editUrl = urlBuilder.getUrl(httpRequest, editUrlParMap);
		return editUrl;
	}

	protected String createAddObjectLink(SourceBean item) {
		HashMap addUrlParMap = new HashMap();
		addUrlParMap.put(Spago4QConstants.PAGE, "MeasureModelKPIPage");
		addUrlParMap.put(Spago4QConstants.MODULE, "ListMeasureModelKPIModule");
		addUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_NEW");
		addUrlParMap.putAll(getItemParams(item));
		String addUrl = urlBuilder.getUrl(httpRequest, addUrlParMap);
		return addUrl;
	}

	protected String createDeleteObjectLink(SourceBean item) {
		HashMap deleteUrlParMap = new HashMap();
		deleteUrlParMap.put(Spago4QConstants.PAGE, "MeasureModelKPIPage");
		deleteUrlParMap.put(Spago4QConstants.MODULE, "ListMeasureModelKPIModule");
		deleteUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, DelegatedBasicListService.LIST_DELETE);
		deleteUrlParMap.putAll(getItemParamsDelete(item));
		String addUrl = urlBuilder.getUrl(httpRequest, deleteUrlParMap);
		return addUrl;
	}




	@Override
	protected Map getItemParams(SourceBean item) {
		Map m = new HashMap();
		int mm = ((Integer) item.getAttribute("id")).intValue();
		String mm_name=(String)item.getAttribute("mm_name");
		int area = ((Integer) item.getAttribute("id_area")).intValue();
		String area_name = (String) item.getAttribute("area_name");
		int mf = ((Integer) item.getAttribute("id_measurement_framework")).intValue();
		String mf_name = (String) item.getAttribute("mf_name");
		int in = ((Integer) item.getAttribute("id_information_need")).intValue();
		String in_name = (String) item.getAttribute("in_name");
		String in_code = (String) item.getAttribute("in_code");
		int mc = ((Integer) item.getAttribute("id_measurable_concept")).intValue();
		String mc_name = (String) item.getAttribute("mc_name");
		String mc_code = (String) item.getAttribute("mc_code");
		int ma = ((Integer) item.getAttribute("id_measurable_attribute")).intValue();
		String ma_name = (String) item.getAttribute("ma_name");
		String ma_code = (String) item.getAttribute("ma_code");
		int kpi = ((Integer) item.getAttribute("id_kpi")).intValue();
		String kpi_name = (String) item.getAttribute("kpi_name");

		String itemType = (String) item.getAttribute("itemType");
		m.put("id", mm);
		m.put("mm_name", mm_name);
		m.put("area", area);
		m.put("area_name", area_name);
		m.put("mf", mf);
		m.put("mf_name", mf_name);
		m.put("in", in);
		m.put("in_name", in_name);
		if(in_code!=null)
			m.put("in_code", in_code);
		m.put("mc", mc);
		m.put("mc_name", mc_name);
		if(mc_code!=null)
			m.put("mc_code", mc_code);
		m.put("ma", ma);
		m.put("ma_name", ma_name);
		if(ma_code!=null)
			m.put("ma_code", ma_code);
		m.put("type", itemType);
		m.put("kpi", kpi);
		m.put("kpi_name", kpi_name);
		return m;
	}

	@Override
	public Map getItemParamsDelete(SourceBean item) {
		Map m = new HashMap();
		int mm = ((Integer) item.getAttribute("id")).intValue();
		int area = ((Integer) item.getAttribute("id_area")).intValue();
		int mf = ((Integer) item.getAttribute("id_measurement_framework")).intValue();
		int in = ((Integer) item.getAttribute("id_information_need")).intValue();
		int mc = ((Integer) item.getAttribute("id_measurable_concept")).intValue();
		int ma = ((Integer) item.getAttribute("id_measurable_attribute")).intValue();
		int kpi = ((Integer) item.getAttribute("id_kpi")).intValue();

		String itemType = (String) item.getAttribute("itemType");
		m.put("mm", mm);
		m.put("area", area);
		m.put("mf", mf);
		m.put("in", in);
		m.put("mc", mc);
		m.put("ma", ma);
		m.put("type", itemType);
		m.put("kpi", kpi);
		return m;
	}

}
