/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.presentation.tags.treehtmlgenerators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.list.basic.impl.DelegatedBasicListService;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago4q.utilities.Spago4QConstants;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class MeasureStructureTreeHtmlGenerator extends StructureTreeHtmlGenerator  {

	int mf_ = -1;
	int in_ = -1;
	int mc_ = -1;
	int ma_ = -1;

	int id_mf_ = -1;
	int id_in_ = -1;
	int id_mc_ = -1;
	int id_ma_ = -1;

	@Override
	protected String getTreeName(List objectsList) {
		try {
			SourceBean sb = (SourceBean) objectsList.get(0);
			return (String) sb.getAttribute("mf_name");
		} catch (Exception e) {
			return "";
		}
	}

	@Override
	protected void addItems(StringBuffer htmlStream, List oldItems,
			String initialPath) {

		int index = -1;

		List items=completeList(oldItems);

		Iterator it = items.iterator();	

		while (it.hasNext()) {

			SourceBean item = (SourceBean) it.next();

			int mf=-1;
			int in=-1;
			int mc=-1;
			int ma=-1;

						
			if(item.getAttribute("mf")!=null)
				mf=(Integer.valueOf((String)item.getAttribute("mf"))).intValue();

			if(item.getAttribute("in")!=null)
				in=(Integer.valueOf((String)item.getAttribute("in"))).intValue();

			if(item.getAttribute("mc")!=null)
				mc=(Integer.valueOf((String)item.getAttribute("mc"))).intValue();
			
			if(item.getAttribute("ma")!=null)
				ma=(Integer.valueOf((String)item.getAttribute("ma"))).intValue();

			index = index + 1;
			String name = "";
			String itemType = "";
			boolean isInitialPath=false;
			boolean isFinalPath=false;

			int parentId = DTREE_ROOT_ID;

			if (mf_ == -1 || mf != mf_){
				mf_ = mf;
				in_ = -1;
				mc_ = -1;
				ma_ = -1;
				id_mf_ = index;
				parentId = -1;
				itemType = "mf";
				name = (String) item.getAttribute("mf_name");
				isInitialPath=true;
			}

			else if (in_ == -1 || in != in_){
				in_ = in;
				mc_ = -1;
				ma_ = -1;
				id_in_ = index;
				parentId = id_mf_;
				itemType = "in";
				name = (String) item.getAttribute("in_name");
			}

			else if (mc_ == -1 || mc != mc_){
				mc_ = mc;
				ma_ = -1;
				id_mc_ = index;
				parentId = id_in_;
				itemType = "mc";
				name = (String) item.getAttribute("mc_name");
			}

			else if (ma_ == -1 || ma != ma_){
				ma_ = ma;
				id_ma_ = index;
				parentId = id_mc_;
				itemType = "ma";
				name = (String) item.getAttribute("ma_name");
				isFinalPath=true;
			}

			try {
				item.updAttribute("itemType", itemType);
			} catch (SourceBeanException e) {
				e.printStackTrace();
			}

			addItemForJSTree(htmlStream, item, index, parentId, name, isInitialPath, isFinalPath);


		}
	}

	protected String createDetailObjectLink(SourceBean item) {
		HashMap editUrlParMap = new HashMap();
		editUrlParMap.put(Spago4QConstants.PAGE, "MeasurementFrameworkTreePage");
		editUrlParMap.put(Spago4QConstants.MODULE, "ListMeasurementFrameworkTreeModule");
		editUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_SELECT");
		editUrlParMap.putAll(getItemParams(item));
		String editUrl = urlBuilder.getUrl(httpRequest, editUrlParMap);
		return editUrl;
	}

	protected String createAddObjectLink(SourceBean item) {
		HashMap addUrlParMap = new HashMap();
		addUrlParMap.put(Spago4QConstants.PAGE, "MeasurementFrameworkTreePage");
		addUrlParMap.put(Spago4QConstants.MODULE, "ListMeasurementFrameworkTreeModule");
		addUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, "DETAIL_NEW");
		addUrlParMap.putAll(getItemParams(item));
		String addUrl = urlBuilder.getUrl(httpRequest, addUrlParMap);
		return addUrl;
	}

	protected String createDeleteObjectLink(SourceBean item) {
		HashMap deleteUrlParMap = new HashMap();
		deleteUrlParMap.put(Spago4QConstants.PAGE, "MeasurementFrameworkTreePage");
		deleteUrlParMap.put(Spago4QConstants.MODULE, "ListMeasurementFrameworkTreeModule");
		deleteUrlParMap.put(Spago4QConstants.PAGE_MESSAGE, DelegatedBasicListService.LIST_DELETE);
		deleteUrlParMap.putAll(getItemParamsDelete(item));
		String addUrl = urlBuilder.getUrl(httpRequest, deleteUrlParMap);
		return addUrl;
	}




	@Override
	protected Map getItemParams(SourceBean item) {
		Map m = new HashMap();

		Integer mf=-1;
		Integer in=-1;
		Integer mc=-1;
		Integer ma=-1;
		String mf_name="";
		String in_name="";
		String mc_name="";
		String ma_name="";
		String in_code="";
		String mc_code="";
		String ma_code="";

		if(item.getAttribute("mf")!=null)
			mf = Integer.valueOf((String) item.getAttribute("mf")).intValue();

		if(item.getAttribute("mf_name")!=null)
			mf_name = (String) item.getAttribute("mf_name");

		if(item.getAttribute("in")!=null)
			in = Integer.valueOf((String) item.getAttribute("in")).intValue();

		if(item.getAttribute("in_name")!=null)
			in_name = (String) item.getAttribute("in_name");

		if(item.getAttribute("in_code")!=null)
			in_code = (String) item.getAttribute("in_code");

		if(item.getAttribute("mc")!=null)
			mc = Integer.valueOf((String) item.getAttribute("mc")).intValue();

		if(item.getAttribute("mc_name")!=null)	
			mc_name = (String) item.getAttribute("mc_name");

		if(item.getAttribute("mc_code")!=null)	
			mc_code = (String) item.getAttribute("mc_code");

		if(item.getAttribute("ma")!=null)	
			ma = Integer.valueOf((String) item.getAttribute("ma")).intValue();

		if(item.getAttribute("ma_name")!=null)
			ma_name = (String) item.getAttribute("ma_name");

		if(item.getAttribute("ma_code")!=null)	
			ma_code = (String) item.getAttribute("ma_code");

		String itemType = (String) item.getAttribute("itemType");
		m.put("mf", mf.intValue());
		m.put("mf_name", mf_name);
		m.put("in", in.intValue());
		m.put("in_name", in_name);
		m.put("in_code", in_code);
		m.put("mc", mc.intValue());
		m.put("mc_name", mc_name);
		m.put("mc_code", mc_code);
		m.put("ma", ma.intValue());
		m.put("ma_name", ma_name);
		m.put("ma_code", ma_code);
		m.put("type", itemType);

		return m;
	}

	@Override
	public Map getItemParamsDelete(SourceBean item) {
		Map m = new HashMap();

		Integer mf=-1;
		Integer in=-1;
		Integer mc=-1;
		Integer ma=-1;

		if(item.getAttribute("mf")!=null)
			mf = Integer.valueOf((String) item.getAttribute("mf")).intValue();
		if(item.getAttribute("in")!=null)	
			in = Integer.valueOf((String) item.getAttribute("in")).intValue();
		if(item.getAttribute("mc")!=null)	
			mc = Integer.valueOf((String) item.getAttribute("mc")).intValue();
		if(item.getAttribute("ma")!=null)	
			ma = Integer.valueOf((String) item.getAttribute("ma")).intValue();

		String itemType = (String) item.getAttribute("itemType");

		m.put("mf", mf.intValue());
		m.put("in", in.intValue());
		m.put("mc", mc.intValue());
		m.put("ma", ma.intValue());
		m.put("type", itemType);
		return m;
	}


	public List completeList(List items){

		String mf="-1";
		String in="-1";
		String mc="-1";
		String ma="-1";

		String mf_name="";
		String mf_code="";
		String in_name="";
		String in_code="";
		String mc_name="";
		String mc_code="";
		String ma_name="";
		String ma_code="";

		String p=new String();
		String element=new String();

		PaginatorIFace paginator = new GenericPaginator();

		List completed=new ArrayList();
		Vector<String> path=new Vector<String>();
		Iterator it = items.iterator();



		while (it.hasNext()) {

			mf="-1";
			in="-1";
			mc="-1";
			ma="-1";

			mf_name="";
			mf_code="";
			in_name="";
			in_code="";
			mc_name="";
			mc_code="";
			ma_name="";
			ma_code="";


			SourceBean item = (SourceBean) it.next();
			if(item.getAttribute("mf")!=null)
			{
				mf = (String.valueOf(item.getAttribute("mf")));
				if(item.getAttribute("mf_name")!=null)
					mf_name=(String)item.getAttribute("mf_name");
				if(item.getAttribute("mf_code")!=null)
					mf_code=(String)item.getAttribute("mf_code");

				p=mf+"-0-0-0";

				if(!path.contains(p)){

					element=writeRow(mf, mf_name, mf_code, "0", "", "", "0", "", "", "0", "", "");
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);


					completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					path.add(p);

				}
			}

			if(item.getAttribute("in")!=null)
			{
				in = (String.valueOf(item.getAttribute("in")));
				if(item.getAttribute("in_name")!=null)
					in_name=(String)item.getAttribute("in_name");
				if(item.getAttribute("in_code")!=null)
					in_code=(String)item.getAttribute("in_code");

				p=mf+"-"+in+"-0-0";

				if(!path.contains(p)){
					
					element=writeRow(mf, mf_name, mf_code, in, in_name, in_code, "0", "", "", "0", "", "");

					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
			
					path.add(p);

				}
			}

			if(item.getAttribute("mc")!=null)
			{
				mc = (String.valueOf(item.getAttribute("mc")));
				if(item.getAttribute("mc_name")!=null)
					mc_name=(String)item.getAttribute("mc_name");
				if(item.getAttribute("mc_code")!=null)
					mc_code=(String)item.getAttribute("mc_code");

				p=mf+"-"+in+"-"+mc+"-0";

				if(!path.contains(p)){

					element=writeRow(mf, mf_name, mf_code, in, in_name, in_code, mc, mc_name, mc_code, "0", "", "");
					
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					path.add(p);

				}
			}			

			if(item.getAttribute("ma")!=null)
			{
				ma = (String.valueOf(item.getAttribute("ma")));
				if(item.getAttribute("ma_name")!=null)
					ma_name=(String)item.getAttribute("ma_name");
				if(item.getAttribute("ma_code")!=null)
					ma_code=(String)item.getAttribute("ma_code");

				p=mf+"-"+in+"-"+mc+"-"+ma;

				if(!path.contains(p)){


					element=writeRow(mf, mf_name, mf_code, in, in_name, in_code, mc, mc_name, mc_code, ma, ma_name, ma_code);
					SourceBean elementBean;
					try {
						elementBean = SourceBean.fromXMLString(element);
						completed.add(elementBean);
					} catch (SourceBeanException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				
					path.add(p);

				}
			}
		}

		return completed;

	}



	
	public String writeRow(String mf, String mf_name, String mf_code,String in, String in_name, String in_code,
			String mc, String mc_name, String mc_code, String ma, String ma_name, String ma_code){

		String s=new String("");

			s=s+"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";

		s=s+"<ROW mf=\""+mf+"\" mf_name=\""+mf_name+"\" mf_code=\""+mf_code+"\"" + 
		" in=\""+in+"\" in_name=\""+in_name+"\" in_code=\""+in_code+"\""+  
		" mc=\""+mc+"\" mc_name=\""+mc_name+"\" mc_code=\""+mc_code+"\""+  
		" ma=\""+ma+"\" ma_name=\""+ma_name+"\" ma_code=\""+ma_code+"\"/>\n";  

		return s;
	}

}

