/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.services.modules;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.dispatching.module.list.basic.impl.DefaultBasicListModule;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.paginator.basic.ListIFace;

public class ListAssessmentFrameworkTreeModule extends DefaultBasicListModule {


	private final static String assCat="select count(id_assessment_category) as many from dt_model_assess_kpi where id_assessment_category=";
	private final static String assTar="select count(id_assessment_target) as many from dt_model_assess_kpi where id_assessment_target=";
	private final static String assPra="select count(id_assessment_practice) as many from dt_model_assess_kpi where id_assessment_practice=";


	@Override
	public boolean delete(SourceBean request, SourceBean response) {

		if(canDelete()){
		return super.delete(request, response);}
		else
			{
			EMFErrorHandler engErrorHandler = getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "10012"));			
			return false;
			}
	}

	@Override
	public ListIFace getList(SourceBean request, SourceBean response)
	throws Exception {

		return super.getList(request, response);

	}

	
	
	//can delete only if the record is not used in measure_model_kpi
	protected boolean canDelete(){
		String type="";
		String many=new String("1");
		String id="";
		RequestContainer requestContainer = RequestContainer.getRequestContainer();
		if ((String) requestContainer.getServiceRequest().getAttribute("type") != null)
			type = (String) requestContainer.getServiceRequest().getAttribute("type");

		String statement="";
		if(type.equalsIgnoreCase("ac")){
			statement=assCat;
			if ((String) requestContainer.getServiceRequest().getAttribute("ac") != null)
				id = (String) requestContainer.getServiceRequest().getAttribute("ac");
		}
		else if(type.equalsIgnoreCase("at")) {
			statement=assTar;
			if ((String) requestContainer.getServiceRequest().getAttribute("at") != null)
				id = (String) requestContainer.getServiceRequest().getAttribute("at");	
		}
		else if(type.equalsIgnoreCase("ap")) {
			statement=assPra;
			if ((String) requestContainer.getServiceRequest().getAttribute("ap") != null)
				id = (String) requestContainer.getServiceRequest().getAttribute("ap");	
				}

		statement+=id;
		
		DataConnection dataConnection=null;
		ScrollableDataResult scrollableDataResult=null;
		DataResult dataResult=null;
		SQLCommand sqlCommand=null;

		try{

			DataConnectionManager dataConnectionManager=DataConnectionManager.getInstance();
			dataConnection=dataConnectionManager.getConnection("spago4q");

			sqlCommand = dataConnection.createSelectCommand(statement);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			DataRow dataRow=scrollableDataResult.getDataRow();
			DataField dataField=dataRow.getColumn("many");
			many=dataField.getStringValue();
		}
		catch (Exception e) {
		}
		finally{
			try{
				if(scrollableDataResult!=null)
					scrollableDataResult.close();
				if(dataConnection!=null)
					dataConnection.close();
			}catch (Exception e) {
				// TODO: handle exception
			}
		}
		if(many.equalsIgnoreCase("0")) return true;
		else return false;

	}

}