/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.RequestContainerPortletAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.ResponseContainerPortletAccess;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.util.PortletUtilities;

import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;

public class ChannelUtilities {

	public static RequestContainer getRequestContainer(HttpServletRequest httpRequest) {
		RequestContainer reqCont = null;
		// try to find the RequestContainer
		reqCont = RequestContainerPortletAccess.getRequestContainer(httpRequest);
		if (reqCont == null) reqCont = RequestContainerAccess.getRequestContainer(httpRequest);
		return reqCont;
	}
	
	public static ResponseContainer getResponseContainer(HttpServletRequest httpRequest) {
		ResponseContainer respCont = null;
		// try to find the ResponseContainer
		respCont = ResponseContainerPortletAccess.getResponseContainer(httpRequest);
		if (respCont == null) respCont = ResponseContainerAccess.getResponseContainer(httpRequest);
		return respCont;
	}
	
	
	public static String getPreferenceValue(RequestContainer requestContainer, String preferenceName, String defaultValue) {
		String prefValue = defaultValue;
		try{
			// get mode of execution
			String channelType = requestContainer.getChannelType();
			String sbiMode = null;
			if (Spago4QConstants.SPAGO4Q_MODE_PORTLET.equalsIgnoreCase(channelType)) sbiMode = Spago4QConstants.SPAGO4Q_MODE_PORTLET;
			else sbiMode = Spago4QConstants.SPAGO4Q_MODE_WEB;
			// based on mode get spago object and url builder
			if (sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_WEB)) {
				Object attribute = requestContainer.getSessionContainer().getAttribute(preferenceName);
				if (attribute != null) prefValue = attribute.toString();
				else prefValue = defaultValue;
			} else if  (sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_PORTLET)){
				PortletRequest portReq = PortletUtilities.getPortletRequest();
				PortletPreferences prefs = portReq.getPreferences();
				prefValue = (String)prefs.getValue(preferenceName, defaultValue);
			}
		} catch (Exception e) {
			Spago4QTracer.major(Spago4QConstants.NAME_MODULE, ChannelUtilities.class.getName(), 
					            "getPreferenceValue", "Error while recovering preference value", e);
			prefValue = defaultValue;
		}
		return prefValue;
	}
	
	public static boolean isWebRunning() {
		ConfigSingleton spagoconfig = ConfigSingleton.getInstance();
		// get mode of execution
		String sbiMode = (String)spagoconfig.getAttribute(Spago4QConstants.SPAGO4Q_MODE);   
		if( (sbiMode!=null) && sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_WEB) ) {
			return true;
		} else {
			return false;
		}
	}
	
	public static boolean isPortletRunning() {
		ConfigSingleton spagoconfig = ConfigSingleton.getInstance();
		// get mode of execution
		String sbiMode = (String)spagoconfig.getAttribute(Spago4QConstants.SPAGO4Q_MODE);   
		if( (sbiMode==null) || !sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_WEB)){
			return true;
		} else {
			return false;
		}
	}
	
	
}
