/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.messages;

import javax.servlet.http.HttpServletRequest;

/**
 * This is the interface for classes that implements logic
 * to retrieve localized messages to be used in JSP pages 
 */
public interface IMessageBuilder {

	/**
	 * Returns the message associated with code in the default bundle
	 * @param code: the message code
	 * @return: the message associated with code in the default bundle
	 */
	public String getMessage(String code);
	
	/**
	 * Returns the message associated with code in the given bundle
	 * @param code: the message code
	 * @param bundle: the bundle to use
	 * @return  the message associated with code in the given bundle
	 */
	public String getMessage(String code, String bundle);
	
	/**
	 * Returns the message associated with code in the default bundle
	 * @param code: the message code
	 * @param request: the http request
	 * @return: the message associated with code in the default bundle
	 */
	public String getMessage(String code, HttpServletRequest request);
	
	/**
	 * Returns the message associated with code in the given bundle
	 * @param code: the message code
	 * @param bundle: the bundle to use
	 * @param request: the http request
	 * @return  the message associated with code in the given bundle
	 */
	public String getMessage(String code, String bundle, HttpServletRequest request);
	
}
