/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.messages;


import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.util.PortletUtilities;
import it.eng.spago4q.utilities.Spago4QConstants;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
/**
 * The implementation of IMessageBuilder   
 */
public class MessageBuilder implements IMessageBuilder {

	public String getMessage(String code) {
		return getMessageInternal(code, "messages", null);
	}
	
	
	public String getMessage(String code, String bundle) {
		return getMessageInternal(code, bundle, null);
	}
	
	public String getMessage(String code, HttpServletRequest request) {
		return getMessageInternal(code, "messages", request);
	}
	
	
	public String getMessage(String code, String bundle, HttpServletRequest request) {
		return getMessageInternal(code, bundle, request);
	}
	
	
	
	private String getMessageInternal(String code, String bundle, HttpServletRequest request) {
		String message = "";
		ConfigSingleton spagoconfig = ConfigSingleton.getInstance();
		// get mode of execution
		String sbiMode = (String)spagoconfig.getAttribute(Spago4QConstants.SPAGO4Q_MODE);   
		// based on mode get spago object and url builder
		if (sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_WEB)) {
			Locale locale = null;
			if(request==null) {
				locale = getBrowserLocaleFromSpago();
			} else {
				locale = getBrowserLocale(request);
			}
			message = MessageBundle.getMessage(code, bundle, locale);
			if((message==null) || message.trim().equals("")) {
				message = code;
			}
		} else if  (sbiMode.equalsIgnoreCase(Spago4QConstants.SPAGO4Q_MODE_PORTLET)){
			message = PortletUtilities.getMessage(code, bundle);
		}
		return message;
	}
	
	
	
	private Locale getBrowserLocaleFromSpago() {
		Locale browserLocale = null;
		RequestContainer reqCont = RequestContainer.getRequestContainer();
		if(reqCont!=null) {
			Object obj = reqCont.getInternalRequest();
			if(obj != null) {
				if(obj instanceof HttpServletRequest) {
					HttpServletRequest request = (HttpServletRequest)obj;
					Locale reqLocale = request.getLocale();
					String language = reqLocale.getLanguage();
					SourceBean langSB = (SourceBean)ConfigSingleton.getInstance()
                     					.getFilteredSourceBeanAttribute(Spago4QConstants.SPAGO4Q_LANGUAGE_SUPPORTED_LANGUAGE, 
                     		                           					"language", language);
					if(langSB!=null) {
						String country = (String)langSB.getAttribute("country");
						browserLocale = new Locale(language, country);
					}
				}
			}
		}
		if(browserLocale==null) {
			browserLocale = getDefaultLocale();
		}
		return browserLocale;
	}
	
	
	
	
	private Locale getBrowserLocale(HttpServletRequest request) {
		Locale browserLocale = null;
		Locale reqLocale = request.getLocale();
		String language = reqLocale.getLanguage();
		SourceBean langSB = (SourceBean)ConfigSingleton.getInstance()
                    					.getFilteredSourceBeanAttribute(Spago4QConstants.SPAGO4Q_LANGUAGE_SUPPORTED_LANGUAGE, 
                     		                           					"language", language);
		if(langSB!=null) {
			String country = (String)langSB.getAttribute("country");
			browserLocale = new Locale(language, country);
		}
		if(browserLocale==null) {
			browserLocale = getDefaultLocale();
		}
		return browserLocale;
	}
	
	
	
	
	private Locale getDefaultLocale() {
	 	// get the configuration sourceBean/language code/country code of the default language
	 	SourceBean defaultLangSB = (SourceBean)ConfigSingleton.getInstance()
	 	                           .getFilteredSourceBeanAttribute(Spago4QConstants.SPAGO4Q_LANGUAGE_SUPPORTED_LANGUAGE, 
	 	                           		                           "default", "true");
	 	String defaultLang = (String)defaultLangSB.getAttribute("language");
	 	String defaultCountry = (String)defaultLangSB.getAttribute("country");
	 	// create the locale
 		Locale locale = new Locale(defaultLang, defaultCountry);
	 	return locale;
	}
	
}
