/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.modelKpiUtilities;

import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

import java.util.Vector;


/*
 * This class provides functions for the costruction of the tree in the assessment model
 * 
 */

public class AlreadyAssignedAssessmentUtil {

	protected DataConnection dataConnection=null;
	protected DataConnectionManager dataConnectionManager=null;
	protected SQLCommand sqlCommand = null;
	protected DataResult dataResult = null;
	protected Vector<String> assignedIds=null;
	protected Boolean none=null;

	protected String id_model_assess=null;
	protected String id_assessment_framework=null;
	protected String id_assessment_category=null;
	protected String id_assessment_target=null;

	private static final String queryAssFrame="SELECT DISTINCT id_assessment_framework as id FROM dt_model_assess_kpi";
	private static final String queryAc="SELECT DISTINCT id_assessment_category as id FROM dt_model_assess_kpi";
	private static final String queryAt="SELECT DISTINCT id_assessment_target as id FROM dt_model_assess_kpi";
	private static final String queryAp="SELECT DISTINCT id_assessment_practice as id FROM dt_model_assess_kpi";
	private static final String leftAf="SELECT id_assessment_framework as id FROM dt_assessment_framework";
	private static final String leftAc="SELECT id_assessment_category as id FROM dt_assessment_category";
	private static final String leftAt="SELECT id_assessment_target as id FROM dt_assessment_target";
	private static final String leftAp="SELECT id_assessment_practice as id FROM dt_assessment_practice";

	
	
	public AlreadyAssignedAssessmentUtil(String _id_model_assess, String _id_assessment_framework, String _id_assessment_category, String _id_assessment_target)
	throws EMFInternalError {
		dataConnectionManager=DataConnectionManager.getInstance();
		dataConnection=dataConnectionManager.getConnection("spago4q");
		id_model_assess=_id_model_assess;
		id_assessment_framework=_id_assessment_framework;
		id_assessment_category=_id_assessment_category;
		id_assessment_target=_id_assessment_target;
		assignedIds=new Vector<String>();
		none=new Boolean(false);
	}


	private void getAssignedIds(String type){
		String statement="";
		if(type.equalsIgnoreCase("ma")){
			statement=queryAssFrame;
			statement=statement+" WHERE id_assessment_framework!=0 AND id_model_assess="+id_model_assess;
		}
		else if(type.equalsIgnoreCase("af")){
			statement = queryAc;
			statement=statement+" WHERE id_assessment_category!=0 AND id_model_assess="+id_model_assess+" AND id_assessment_framework="+id_assessment_framework;
					}
		else if(type.equalsIgnoreCase("ac")){
			statement = queryAt;
			statement=statement+" WHERE id_assessment_target!=0 AND id_model_assess="+id_model_assess+" AND id_assessment_framework="+id_assessment_framework+" AND id_assessment_category="+id_assessment_category;

		}
		else if(type.equalsIgnoreCase("at")){
			statement = queryAp;
			statement=statement+" WHERE id_assessment_practice!=0 AND id_model_assess="+id_model_assess+" AND id_assessment_framework="+id_assessment_framework+" AND id_assessment_category="+id_assessment_category+" AND id_assessment_target="+id_assessment_target;
		}
		
		sqlCommand = dataConnection.createSelectCommand(statement);
		try {
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			assignedIds=new Vector<String>();		
			for(int i=0;i<scrollableDataResult.getRowsNumber();i++){
				DataRow dataRow=scrollableDataResult.getDataRow();
				DataField idF=dataRow.getColumn("id");
				String id=idF.getStringValue();
				assignedIds.add(id);
			}
		} catch (EMFInternalError e) {
			e.printStackTrace();
		}		
	}


	public void areLeftIds(String type) {
		String statement="";
		if(type.equalsIgnoreCase("ma")){
			statement=leftAf;
			statement+=" WHERE id_assessment_framework!=0";
		}
		else if(type.equalsIgnoreCase("af")){
			statement=leftAc;
			statement+=" WHERE id_assessment_category!=0 AND id_assessment_framework="+id_assessment_framework;
		}
		else if(type.equalsIgnoreCase("ac")){
			statement=leftAt;
			statement+=" WHERE id_assessment_target!=0 AND id_assessment_category="+id_assessment_category;
		}
		else if(type.equalsIgnoreCase("at")){
			statement=leftAp;
			statement+=" WHERE id_assessment_practice!=0 AND id_assessment_target="+id_assessment_target;
		}

		sqlCommand = dataConnection.createSelectCommand(statement);
		Vector<String> leftIds=new Vector<String>();
		try {
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			leftIds=new Vector<String>();		
			for(int i=0;i<scrollableDataResult.getRowsNumber();i++){
				DataRow dataRow=scrollableDataResult.getDataRow();
				DataField idF=dataRow.getColumn("id");
				String id=idF.getStringValue();
				leftIds.add(id);
			}
			if(assignedIds.size()>=leftIds.size()){
				none=true;
			}
			else {none=false;}
		} 
		catch (EMFInternalError e) {
			e.printStackTrace();
		}
	}	



	public void runFunctions(String type) throws EMFInternalError{
		assignedIds=new Vector<String>();
		try{
		getAssignedIds(type);
		areLeftIds(type);
		}	
		finally{
			dataConnection.close();
		}
	}

	public Vector<String> getAssignedIds() {
		return assignedIds;
	}

	public Boolean getNone() {
		return none;
	}


}
