/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.modelKpiUtilities;

import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

import java.util.Vector;

public class AlreadyAssignedMeasureUtil {

	protected DataConnection dataConnection=null;
	protected DataConnectionManager dataConnectionManager=null;
	protected SQLCommand sqlCommand = null;
	protected DataResult dataResult = null;
	protected Vector<String> assignedIds=null;
	protected Boolean none=null;

	protected String id_model_measure=null;
	protected String id_area=null;
	protected String id_measurement_framework=null;
	protected String id_information_need=null;
	protected String id_measurable_concept=null;

	private static final String queryMeasFrame="SELECT DISTINCT id_measurement_framework as id FROM dt_model_measure_kpi";
	private static final String queryIn="SELECT DISTINCT id_information_need as id FROM dt_model_measure_kpi";
	private static final String queryMc="SELECT DISTINCT id_measurable_concept as id FROM dt_model_measure_kpi";
	private static final String queryMa="SELECT DISTINCT id_measurable_attribute as id FROM dt_model_measure_kpi";
	private static final String queryArea="SELECT DISTINCT id_area as id FROM dt_model_measure_kpi";
	private static final String leftArea="SELECT id_area as id FROM dt_area";
	private static final String leftMf="SELECT id_measurement_framework as id FROM dt_measurement_framework";
	private static final String leftIn="SELECT id_information_need as id FROM dt_information_need";
	private static final String leftMc="SELECT id_measurable_concept as id FROM dt_measurable_concept";
	private static final String leftMa="SELECT id_measurable_attribute as id FROM dt_measurable_attribute";


	
	
	public AlreadyAssignedMeasureUtil(String _id_model_measure, String _id_area, String _id_measurement_framework, String _id_information_need, String _id_measurable_concept)
	throws EMFInternalError {
		dataConnectionManager=DataConnectionManager.getInstance();
		dataConnection=dataConnectionManager.getConnection("spago4q");
		id_model_measure=_id_model_measure;
		id_area=_id_area;
		id_measurement_framework=_id_measurement_framework;
		id_information_need=_id_information_need;
		id_measurable_concept=_id_measurable_concept;
		assignedIds=new Vector<String>();
		none=new Boolean(false);
	}


	private void getAssignedIds(String type){
		String statement="";
		if(type.equalsIgnoreCase("mm")){
			statement=queryArea;
			statement=statement+" WHERE id_area!=0 AND id_model_measure="+id_model_measure;
		}
		else if(type.equalsIgnoreCase("area")){
			statement = queryMeasFrame;
			statement=statement+" WHERE id_measurement_framework!=0 AND id_model_measure="+id_model_measure+" AND id_area="+id_area;
		}
		else if(type.equalsIgnoreCase("mf")){
			statement = queryIn;
			statement=statement+" WHERE id_information_need!=0 AND id_model_measure="+id_model_measure+" AND id_area="+id_area+" AND id_measurement_framework="+id_measurement_framework;
					}
		else if(type.equalsIgnoreCase("in")){
			statement = queryMc;
			statement=statement+" WHERE id_measurable_concept!=0 AND id_model_measure="+id_model_measure+" AND id_area="+id_area+" AND id_measurement_framework="+id_measurement_framework+" AND id_information_need="+id_information_need;

		}
		else if(type.equalsIgnoreCase("mc")){
			statement = queryMa;
			statement=statement+" WHERE id_measurable_attribute!=0 AND id_model_measure="+id_model_measure+" AND id_area="+id_area+" AND id_measurement_framework="+id_measurement_framework+" AND id_information_need="+id_information_need+" AND id_measurable_concept="+id_measurable_concept;
		}
		
		sqlCommand = dataConnection.createSelectCommand(statement);
		try {
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			assignedIds=new Vector<String>();		
			for(int i=0;i<scrollableDataResult.getRowsNumber();i++){
				DataRow dataRow=scrollableDataResult.getDataRow();
				DataField idF=dataRow.getColumn("id");
				String id=idF.getStringValue();
				assignedIds.add(id);
			}
		} catch (EMFInternalError e) {
			e.printStackTrace();
		}		
	}


	public void areLeftIds(String type) {
		String statement="";
		if(type.equalsIgnoreCase("mm")){
			statement=leftArea;
			statement+=" WHERE id_area!=0";
		}
		else if(type.equalsIgnoreCase("area")){
			statement=leftMf;
			statement+=" WHERE id_measurement_framework!=0";
		}
		else if(type.equalsIgnoreCase("mf")){
			statement=leftIn;
			statement+=" WHERE id_information_need!=0 AND id_measurement_framework="+id_measurement_framework;
		}
		else if(type.equalsIgnoreCase("in")){
			statement=leftMc;
			statement+=" WHERE id_measurable_concept!=0 AND id_information_need="+id_information_need;
		}
		else if(type.equalsIgnoreCase("mc")){
			statement=leftMa;
			statement+=" WHERE id_measurable_attribute!=0 AND id_measurable_concept="+id_measurable_concept;
		}

		sqlCommand = dataConnection.createSelectCommand(statement);
		Vector<String> leftIds=new Vector<String>();
		try {
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
			leftIds=new Vector<String>();		
			for(int i=0;i<scrollableDataResult.getRowsNumber();i++){
				DataRow dataRow=scrollableDataResult.getDataRow();
				DataField idF=dataRow.getColumn("id");
				String id=idF.getStringValue();
				leftIds.add(id);
			}
			if(assignedIds.size()>=leftIds.size()){
				none=true;
			}
			else {none=false;}
			if(scrollableDataResult!=null)
				scrollableDataResult.close();
		} 
		catch (EMFInternalError e) {
			e.printStackTrace();
		}
	}	



	public void runFunctions(String type) throws EMFInternalError{
		assignedIds=new Vector<String>();
		try{
		getAssignedIds(type);
		areLeftIds(type);
		}	
		finally{
			dataConnection.close();
		}
	}

	public Vector<String> getAssignedIds() {
		return assignedIds;
	}

	public Boolean getNone() {
		return none;
	}


}
