/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.parameterUtilities;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.InformationDataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.security.DefaultCipher;

import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

public class StoreParameterUtilities {

	protected HttpServletRequest httpRequest = null;
	protected DataConnection dataConnection=null;
	protected SQLCommand sqlCommand = null;
	protected DataResult dataResult = null;
	protected ScrollableDataResult scrollableDataResult=null;



	public Boolean storeDataSourceParameter(it.eng.spago.base.SourceBean request,String type_id,String e_data_source_id){
		Boolean ok=true;
		Boolean finalOk=true;
		DataConnectionManager dataConnectionManager;

		try{
			dataConnectionManager = DataConnectionManager.getInstance();
			dataConnection=dataConnectionManager.getConnection("spago4q");

			HashMap<String, Boolean> parNames=getNames(type_id, "DataSource");

			for (Iterator iterator = parNames.keySet().iterator(); iterator.hasNext();) {
				String name = (String) iterator.next();
				Boolean enc=parNames.get(name);
				String value="";
				if(request.getAttribute(name)!=null)
					value=(String)request.getAttribute(name);
				value.replaceAll("/", "//");

				if(enc.equals(true)){
					//ENcrypt password only if it is different from the one before
					if(request.getAttribute("old"+name)!=null){
					String oldatt=(String)request.getAttribute("old"+name);
					if(!oldatt.equals(value))
					{DefaultCipher cipher=new DefaultCipher();
					value=cipher.encrypt(value);}
					}
					}
				
				Boolean present=isPresent(e_data_source_id, name, "DataSource");


				if(present){
					ok=updateParameter(e_data_source_id,name,value,"DataSource");
				}
				else{
					ok=insertParameter(e_data_source_id, name, value, "DataSource");
				}
				if(ok.equals(false))finalOk=true;
			}
		}
		catch (EMFInternalError e) {
			ok=false;
			finalOk=false;
		}
		finally{
			try{
				if(dataConnection!=null)dataConnection.close();
				if(scrollableDataResult!=null)scrollableDataResult.close();
			}
			catch (EMFInternalError e) {
			}
		}

		return finalOk;
	}	

	public Boolean storeOperationParameter(SourceBean request, String type_id,String e_operation_id) throws EMFInternalError{

		Boolean ok=true;
		Boolean finalOk=true;
		DataConnectionManager dataConnectionManager;

		try{
			dataConnectionManager = DataConnectionManager.getInstance();
			dataConnection=dataConnectionManager.getConnection("spago4q");
			HashMap<String, Boolean> parNames=getNames(type_id, "Operation");

			for (Iterator iterator = parNames.keySet().iterator(); iterator.hasNext();) {
				String name = (String) iterator.next();
				String value="";
				if(request.getAttribute(name)!=null)
					value=(String)request.getAttribute(name);
			
			
			Boolean present=isPresent(e_operation_id, name, "Operation");


			if(present){
				ok=updateParameter(e_operation_id,name,value,"Operation");
			}
			else{
				ok=insertParameter(e_operation_id, name, value, "Operation");
			}
			if(ok.equals(false))finalOk=true;
			}
		}
		catch (EMFInternalError e) {
			ok=false;
			finalOk=false;
		}
		finally{
			try{
				if(dataConnection!=null)dataConnection.close();
				if(scrollableDataResult!=null)scrollableDataResult.close();
			}
			catch (EMFInternalError e) {
			}
		}
		return ok;
	}




	public Boolean isPresent(String id, String name, String type) throws EMFInternalError{

		String statement="";
		String count="1";
		if(type.equalsIgnoreCase("DataSource")){
			statement="Select count(name) as countn from e_data_source_parameters where e_data_source_id=";
		}
		else if(type.equalsIgnoreCase("Operation")){
			statement="Select count(name) as countn from e_operation_parameter where e_operation_id=";
		}
		statement=statement+id+" AND name='"+name+"'";

		sqlCommand = dataConnection.createSelectCommand(statement);
		dataResult = sqlCommand.execute();	
		scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();
		DataRow dataRow=scrollableDataResult.getDataRow();
		DataField countF=dataRow.getColumn("countn");
		count=countF.getStringValue();

		if(Integer.valueOf(count)>0){return true;}
		else {return false;}
	}


	public Boolean updateParameter(String id, String name,String value, String type) throws EMFInternalError {
		String statement="";
		Boolean ok;
		if(type.equalsIgnoreCase("DataSource")){
			statement="UPDATE e_data_source_parameters SET value='"+value+"' WHERE e_data_source_id="+id+" AND name='"+name+"'";
		}
		else if(type.equalsIgnoreCase("Operation")){
			statement="UPDATE e_operation_parameter SET value='"+value+"' WHERE e_data_source_id="+id+" AND name='"+name+"'";
		}

			sqlCommand = dataConnection.createUpdateCommand(statement);
			DataResult info= sqlCommand.execute();	
			ok=((InformationDataResult)info.getDataObject()).correctExecution();

		return ok;

	}

	public Boolean insertParameter(String id, String name,String value, String type) throws EMFInternalError {
		String statement="";
		Boolean ok;
		if(type.equalsIgnoreCase("DataSource")){
			statement="INSERT into e_data_source_parameters(e_data_source_id,name,value) values("+id+",'"+name+"','"+value+"')";
		}
		else if(type.equalsIgnoreCase("Operation")){
			statement="INSERT into e_operation_parameter(e_operation_id,name,value) values("+id+",'"+name+"','"+value+"')";
		}

			sqlCommand = dataConnection.createInsertCommand(statement);
			DataResult info= sqlCommand.execute();	
			ok=((InformationDataResult)info.getDataObject()).correctExecution();


		return ok;

	}

	public HashMap<String, Boolean> getNames(String id, String type) throws EMFInternalError{
		HashMap<String, Boolean> parNames=new HashMap<String, Boolean>();
		String statement="";
		if(type.equalsIgnoreCase("DataSource")){
			statement="SELECT name_parameter,encryption FROM e_source_type_parameter WHERE id_source_type=";
		}
		else if(type.equalsIgnoreCase("Operation")){
			statement="SELECT name_parameter FROM e_source_type_operation_parameter WHERE id_source_type=";
		}
		statement+=id;
		sqlCommand = dataConnection.createSelectCommand(statement);
		dataResult = sqlCommand.execute();	
		scrollableDataResult = (ScrollableDataResult)dataResult.getDataObject();

		DataRow dataRow;
		DataField dataField;
		String name;
		Boolean enc=false;
		while(scrollableDataResult.hasRows()){
			enc=false;
			dataRow=scrollableDataResult.getDataRow();
			dataField=dataRow.getColumn("name_parameter");
			name=dataField.getStringValue();
			if(type.equalsIgnoreCase("DataSource")){
				dataField=dataRow.getColumn("encryption");
				enc=new Boolean(dataField.getStringValue());
			}
			
			parNames.put(name,enc);
		}

		return parNames;
	}



}



