/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.spago4q.utilities.sourceUtilities;

import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class AlreadyAssignedOperationUtil {

	protected DataConnection dataConnection=null;
	protected DataConnectionManager dataConnectionManager=null;
	protected SQLCommand sqlCommand = null;
	protected DataResult dataResult = null;
	protected ScrollableDataResult scrollableDataResult= null;
	protected DataRow dataRow=null;
	protected DataField dataField=null;
	protected String source_type="";
	protected String interface_id="";
	protected Vector<String> namesField=null;
	protected HashMap<String, Boolean> namesParameter=null;


	private static final String getInterface = "select interface_id from e_extraction_process ";

	private static final String getNameFields = "select name_field as name from e_interface_field ";
	private static final String howManyFieldsAlready = "select count(e_operation_id) as many from e_operation_field ";
	private static final String getNamesParamaters = "select name_parameter as name,multiple from e_source_type_operation_parameter ";
	private static final String howManyParametersAlready = "select count(e_operation_id) as many from e_operation_parameter ";
	private static final String howManyParameterName = "select count(name) as many from e_operation_parameter";
	private static final String getSourceTypeOperation = "select eds.type_id as source_type, est.name as source_type_name from e_operation as eop,e_data_source as eds,e_source_type as est where eop.datasource_id=eds.id AND eds.type_id=est.id ";
	private static final String isMultiple = "select multiple from e_source_type_operation_parameter ";

	/*
This class provides methods to assign parameters to operations
	 */
	public AlreadyAssignedOperationUtil() {
		try {
			dataConnectionManager = DataConnectionManager.getInstance();

			dataConnection = dataConnectionManager.getConnection("spago4q");
		}
		catch (EMFInternalError e) {
			e.printStackTrace();
		}
	}


	public String getInterfaceProcess(String id_extraction_process){
		interface_id=new String();
		try{
			String statInt = getInterface + "WHERE id="+ id_extraction_process;

			sqlCommand = dataConnection.createSelectCommand(statInt);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
			DataRow dataRow = scrollableDataResult.getDataRow();
			DataField dataField = dataRow.getColumn("interface_id");
			interface_id = dataField.getStringValue();
		}
		catch (Exception e) {
		}		
		return interface_id;
	}


	public String getSourceTypeOperation(String id_operation){
		source_type=new String();
		String name="";

		String statType = getSourceTypeOperation + "AND eop.id="+id_operation;
		try{
			sqlCommand = dataConnection.createSelectCommand(statType);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
			DataRow dataRow = scrollableDataResult.getDataRow();
			DataField dataField = dataRow.getColumn("source_type");
			source_type = dataField.getStringValue();
			dataField=dataRow.getColumn("source_type_name");
			name=dataField.getStringValue();
		}
		catch (Exception e) {
		}
		return name;

	}

	public String getSourceTypeIdOperation(String id_operation){
		source_type=new String();

		String statType = getSourceTypeOperation + "AND eop.id="+id_operation;
		try{
			sqlCommand = dataConnection.createSelectCommand(statType);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
			DataRow dataRow = scrollableDataResult.getDataRow();
			DataField dataField = dataRow.getColumn("source_type");
			source_type = dataField.getStringValue();
		}
		catch (Exception e) {
		}
		return source_type;

	}




	public void getFieldNames() {
		namesField=new Vector<String>();
		String statement1="";
		try{
			statement1 = getNameFields + " WHERE id_interface=" + interface_id;
			sqlCommand = dataConnection.createSelectCommand(statement1);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult
			.getDataObject();

			// First I get the parameters
			for (int i = 0; i < scrollableDataResult.getRowsNumber(); i++) {
				dataRow = scrollableDataResult.getDataRow();
				dataField= dataRow.getColumn("name");
				String name = dataField.getStringValue();
				namesField.add(name);
			}
		}
		catch (Exception e) {
		}
	}

	public void getParametersNames() {
		namesParameter=new HashMap<String, Boolean>();
		String statement1="";
		try{
			statement1 = getNamesParamaters+ " WHERE id_source_type=" + source_type;	
			sqlCommand = dataConnection.createSelectCommand(statement1);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult
			.getDataObject();

			// First I get the parameters
			for (int i = 0; i < scrollableDataResult.getRowsNumber(); i++) {
				dataRow = scrollableDataResult.getDataRow();
				dataField= dataRow.getColumn("name");
				String name = dataField.getStringValue();
				dataField=dataRow.getColumn("multiple");
				String multiple=dataField.getStringValue();
				namesParameter.put(name,new Boolean(multiple));
			}
		}
		catch (Exception e) {
		}
	}



	public Vector<String> getAvalaibleParametersNames(String id_operation) {

		this.getParametersNames();

		Vector<String> avalaibleNames=new Vector<String>();
		String statement="";

		statement = howManyParametersAlready;	
		statement=statement+ "where e_operation_id="+ id_operation+" AND ";		

		String currStatement="";

		try{

			for (Iterator iterator = namesParameter.keySet().iterator(); iterator.hasNext();) {
				String name = (String) iterator.next();
				Boolean multiple=(Boolean)namesParameter.get(name);

				if(multiple.equals(true)){
					avalaibleNames.add(name);
				}
				else{
					currStatement = statement + "name='" + name + "'";
				

				sqlCommand = dataConnection.createSelectCommand(currStatement);
				dataResult = sqlCommand.execute();
				scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
				dataRow = scrollableDataResult.getDataRow();
				dataField = dataRow.getColumn("many");
				String manyS = dataField.getStringValue();
				if (manyS.equalsIgnoreCase("0")) {
					avalaibleNames.add(name);
				}
				}
				}

		}
		catch (Exception e) {
		}
		return avalaibleNames;
	}


	public Vector<String> getAvalaibleFieldsNames(String id_operation) {

		this.getFieldNames();

		Vector<String> avalaibleNames=new Vector<String>();
		String statement="";

		statement = howManyFieldsAlready;
		statement=statement+ "where e_operation_id="+ id_operation+" AND ";		

		String currStatement;

		try{

			for (Iterator iterator = namesField.iterator(); iterator
			.hasNext();) {
				String name = (String) iterator.next();

				currStatement = statement + "interfaceposition='" + name + "'";

				sqlCommand = dataConnection.createSelectCommand(currStatement);
				dataResult = sqlCommand.execute();
				scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
				dataRow = scrollableDataResult.getDataRow();
				dataField = dataRow.getColumn("many");
				String manyS = dataField.getStringValue();
				if (manyS.equalsIgnoreCase("0")) {
					avalaibleNames.add(name);
				}

			}

		}
		catch (Exception e) {
		}
		return avalaibleNames;
	}







	public void close(){
		try {
			if(scrollableDataResult!=null) scrollableDataResult.close();
			if(dataConnection!=null)dataConnection.close();
		} catch (EMFInternalError e) {

			e.printStackTrace();
		}
	}


	public  Boolean isMultiple(String name,String id_operation){		
		String sourceType=getSourceTypeIdOperation(id_operation);
		String query=isMultiple+"WHERE name_parameter='"+name+"' AND id_source_type="+sourceType;
		String isMultiple="";
		try{
			sqlCommand = dataConnection.createSelectCommand(query);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult
			.getDataObject();
			dataRow = scrollableDataResult.getDataRow();
			dataField = dataRow.getColumn("multiple");
			isMultiple = dataField.getStringValue();
		}
		catch (Exception e) {
		}	

		return new Boolean(isMultiple);
	}


	public Boolean isPresent(String name, String id_operation){
		String manyS="";
		String statement=howManyParameterName+" where e_operation_id="+id_operation+" AND name='"+name+"'";

		try{
			sqlCommand = dataConnection.createSelectCommand(statement);
			dataResult = sqlCommand.execute();
			scrollableDataResult = (ScrollableDataResult) dataResult
			.getDataObject();
			dataRow = scrollableDataResult.getDataRow();
			dataField = dataRow.getColumn("many");
			manyS = dataField.getStringValue();
		}
		catch (Exception e) {
		}

		if (manyS.equalsIgnoreCase("0")) {
			return false;
		}
		else {
			return true;
		}

	}

}

